# ********************************************************
# Root Access & Logging Module
#
# The goal of this module is to allow users to act as root
# using sudo (or even direct login if they have the password),
# but to make sure that whatever root does is *thoroughly*
# logged.  We log 3 different ways; there was never a script
# kiddie born that'll find all of them.
#
# This module goes with
# http://teddyb.org/rlp/tiki-index.php?page=Writeup+Of+My+13+Jan+2009+Ownage
#
# THINGS WE COULD DO (but haven't):
#
# 	The big lose here is that all the data is local,
#	so if you're owned by someone who actually *is*
#	capable of understanding all the systems in question,
#	you're screwed.  Having all the logs delivered off-host
#	would be nice.
#
# ********************************************************
class root {
    # ******************
    # Basic Sudo Setup
    # ******************
    package { sudo: ensure => latest }

    file { "/etc/sudoers":
	owner   => root,
	group   => root,
	mode    => 440,
	source  => "puppet:///root/sudoers",
	require => Package["sudo"],
    }

    # ******************
    # Process Accounting
    #
    # The results of this logging can be reviewed using "sa", "ac",
    # and "lastcomm", all of which are horribly unreadable, although
    # lastcomm is better than the other two IMO.
    # ******************
    package { acct: ensure => latest }

    # ******************
    # Modified Root Shell
    # ******************

    # Get a bunch of static shells because hey, why not.
    package {
	[ "e2fsck-static", "bash-static", "busybox-static",
	    "zsh-static", "sash" ]:
	ensure => latest
    }

    # Shut busybox up; see
    # http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=334672
    #
    # An empty file does fine.
    file { "/etc/busybox.conf":
	owner   => root,
	group   => root,
	mode    => 644,
    }

    # Set up the log directory for the script outputs
    file { "/var/log/root_shells/":
	owner   => root,
	group   => root,
	mode    => 644,
	ensure => directory,
    }

    # Setup for our modified shells
    define script_shell() {
	# Install the modified shell
	file { "/usr/local/bin/$name":
	    owner => root,
	    group => root,
	    mode => 755,
	    source  => "puppet:///root/$name",
	}
	exec { "/usr/sbin/add-shell /usr/local/bin/$name":
	    require => [ File["/usr/local/bin/$name"], File["/var/log/root_shells/"] ],
	    unless => "grep -q /usr/local/bin/$name /etc/shells",
	}
    }

    # Install all the modified shells
    script_shell{ sbash: }
    script_shell{ sbbsh: }
    script_shell{ ssash: }
    script_shell{ szsh: }

    # We need a direct busybox sh link
    file { "busybox sh":
	name => "/usr/local/bin/sh",
	ensure => "/bin/busybox",
    }

    # Put the root password fixing script in place
    file { "/usr/local/bin/fix_rootpw.sh":
	owner   => root,
	group   => root,
	mode    => 755,
	source  => "puppet:///root/fix_rootpw.sh",
    }

    define scripted_root( $shell ) {
	user { $name:
	    allowdupe => true,
	    shell => $shell,
	    uid => 0,
	    gid => 0,
	    home => "/root",
	    comment => "Root User",
	}

	exec { "/usr/local/bin/fix_rootpw.sh $name":
	    require => User[$name],
	    # Don't bother if there's clearly some kind of password
	    # hash there already
	    unless => "grep -Eq '^$name:[^:]{10}' /etc/shadow",
	}
    }
    # Set up our root accounts
    scripted_root{ broot: shell => "/usr/local/bin/sbash" }
    scripted_root{ zroot: shell => "/usr/local/bin/szsh" }
    scripted_root{ sashroot: shell => "/usr/local/bin/ssash" }

    # And give the regular root user a busybox static scripted shell
    # too, but don't use fix_rootpw.sh on it.
    user { root:
	allowdupe => true,
	shell => "/usr/local/bin/sbbsh",
	uid => 0,
	gid => 0,
	home => "/root",
	comment => "Root User",
    }

    # Reap old root logs, with the default filebucket backup
    tidy { "old root script outputs":
	path => "/var/log/root_shells/",
	recurse => inf,
	# Older than one week
	age => "1w",
    }

    # ******************
    # Snoopy Logger
    # ******************
    package { [ snoopy, "ld.so.preload-manager" ]: ensure => latest }

    # sysklogd does daily log rotation on every listed log, which
    # will help with the fact that auth.log is gonna get *huge*
    package { sysklogd: ensure => latest }

    # Turn Snoopy on for *everything*
    exec { "/usr/sbin/ld.so.preload-manager -y /lib/snoopy.so":
	require => [ Package["ld.so.preload-manager"], Package["snoopy"] ],
	unless => "grep -q /lib/snoopy.so /etc/ld.so.preload",
    }

    # Ignore Snoopy in logcheck
    file { "/etc/logcheck/ignore.d.server/snoopy":
	owner   => root,
	group   => logcheck,
	mode    => 640,
	source  => "puppet:///root/logcheck_ignore",
	require => Package["snoopy"],
    }
}
